import 'package:flutter/material.dart';
import 'package:go_router/go_router.dart';
import 'package:niralanlayout/widgets/animated_sprite_widget.dart';

CustomTransitionPage<T> buildTransition<T>({
  required Widget child,
  required Color color,
  bool flipHorizontally = false,
  String? name,
  Object? arguments,
  String? restorationId,
  LocalKey? key,
}) {
  return CustomTransitionPage<T>(
    child: child,
    transitionsBuilder: (BuildContext context, Animation<double> animation,
        Animation<double> secondaryAnimation, Widget child) {
      return InkRevealWidget(
        animation: animation,
        color: color,
        flipHorizontally: flipHorizontally,
        child: child,
      );
    },
    key: key,
    name: name,
    arguments: arguments,
    restorationId: restorationId,
    transitionDuration: const Duration(milliseconds: 700),
  );
}

class InkRevealWidget extends StatefulWidget {
  final Widget child;

  final Animation<double> animation;

  final Color color;

  final bool flipHorizontally;

  const InkRevealWidget({
    required this.child,
    required this.animation,
    required this.color,
    this.flipHorizontally = false,
    Key? key,
  }) : super(key: key);

  @override
  State<InkRevealWidget> createState() => _InkRevealState();
}

class _InkRevealState extends State<InkRevealWidget> {
  bool _finished = false;

  @override
  void initState() {
    super.initState();

    widget.animation.addStatusListener(_statusListener);
  }

  @override
  void didUpdateWidget(covariant InkRevealWidget oldWidget) {
    if (oldWidget.animation != widget.animation) {
      oldWidget.animation.removeStatusListener(_statusListener);
      widget.animation.addStatusListener(_statusListener);
    }
    super.didUpdateWidget(oldWidget);
  }

  @override
  void dispose() {
    widget.animation.removeStatusListener(_statusListener);
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Stack(
      fit: StackFit.expand,
      children: [
        AnimatedSpriteWidget(
          image: const AssetImage('assets/images/scribble_sprites.png'),
          frameWidth: 250,
          frameHeight: 541,
          frameCount: 12,
          animation: widget.animation,
          color: widget.color,
          flipHorizontally: widget.flipHorizontally,
        ),
        AnimatedOpacity(
          opacity: _finished ? 1 : 0,
          duration: const Duration(milliseconds: 300),
          child: widget.child,
        ),
      ],
    );
  }

  void _statusListener(AnimationStatus status) {
    switch (status) {
      case AnimationStatus.completed:
        setState(() {
          _finished = true;
        });
        break;
      case AnimationStatus.forward:
      case AnimationStatus.dismissed:
      case AnimationStatus.reverse:
        setState(() {
          _finished = false;
        });
        break;
    }
  }
}
